@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -75,6 +75,18 @@ public class StateManager extends AbstractRealtimeJanitor {
 
     private final ConcurrentHashMap<ID, StanzaTransmitter> transmitters = new ConcurrentHashMap<ID, StanzaTransmitter>();
 
+    private boolean isShutDown = false;
+
+    
+    /**
+     * Inform the {@link StateManager} about a shutdown
+     *
+     * @param isShutDown true if the StateManager should be should down
+     */
+    public void shutDown() {
+        this.isShutDown= true;
+    }
+
     /**
      * Retrieves stored {@link RTClientState} or creates a new entry for the given id.
      * @param id The id
@@ -122,32 +134,34 @@ public class StateManager extends AbstractRealtimeJanitor {
      * @param timestamp - The timestamp to compare the lastSeen value to
      */
     public void timeOutStaleStates(long timestamp) {
-        GlobalRealtimeCleanup globalRealtimeCleanup = JSONServiceRegistry.getInstance().getService(GlobalRealtimeCleanup.class);
-        DistributedGroupManager groupManager = JSONServiceRegistry.getInstance().getService(DistributedGroupManager.class);
-        for (RTClientState state : new ArrayList<RTClientState>(states.values())) {
-            ID client = state.getId();
-            Duration inactivity = state.getInactivityDuration();
-            LOG.debug("Client {} is inactive since {} seconds", client, inactivity.getValueInS());
-            if(groupManager != null) {
-                try {
-                    groupManager.setInactivity(client, inactivity);
-                } catch(OXException oxe) {
-                    LOG.error("Error while trying to set inactivity of client {}", client, oxe);
-                }
-            } else {
-                LOG.error("Unable to inform GroupManager about inactivity duration. GroupManagerService is missing!");
-            }
-            if (state.isTimedOut(timestamp)) {
-                /*
-                 * The client timed out: if he'd be still active and was just rerouted to another backend the cleanup would have already
-                 * happened during enrol on the other node. As we reached this code there was no cleanup yet and we still have to do
-                 * it cluster-wide.
-                 */
-                LOG.debug("State for id {} is timed out. Last seen: {}", state.getId(), state.getLastSeen());
-                if(globalRealtimeCleanup != null) {
-                    globalRealtimeCleanup.cleanForId(state.getId());
+        if (!isShutDown) {
+            GlobalRealtimeCleanup globalRealtimeCleanup = JSONServiceRegistry.getInstance().getService(GlobalRealtimeCleanup.class);
+            DistributedGroupManager groupManager = JSONServiceRegistry.getInstance().getService(DistributedGroupManager.class);
+            for (RTClientState state : new ArrayList<RTClientState>(states.values())) {
+                ID client = state.getId();
+                Duration inactivity = state.getInactivityDuration();
+                LOG.debug("Client {} is inactive since {} seconds", client, inactivity.getValueInS());
+                if (groupManager != null) {
+                    try {
+                        groupManager.setInactivity(client, inactivity);
+                    } catch (OXException oxe) {
+                        LOG.error("Error while trying to set inactivity of client {}", client, oxe);
+                    }
                 } else {
-                    LOG.error("Unable to cleanup for id {}. GLobalRealtimeCleanupService is missing!", client);
+                    LOG.error("Unable to inform GroupManager about inactivity duration. GroupManagerService is missing!");
+                }
+                if (state.isTimedOut(timestamp)) {
+                    /*
+                     * The client timed out: if he'd be still active and was just rerouted to another backend the cleanup would have already
+                     * happened during enrol on the other node. As we reached this code there was no cleanup yet and we still have to do
+                     * it cluster-wide.
+                     */
+                    LOG.debug("State for id {} is timed out. Last seen: {}", state.getId(), state.getLastSeen());
+                    if (globalRealtimeCleanup != null) {
+                        globalRealtimeCleanup.cleanForId(state.getId());
+                    } else {
+                        LOG.error("Unable to cleanup for id {}. GLobalRealtimeCleanupService is missing!", client);
+                    }
                 }
             }
         }
